#ifndef pilo_STRUCT_H
#define pilo_STRUCT_H

/* --
   -- Init parameters
   --                                                                */
typedef struct PILO_INIT_STR {
  char posPosterName[64];   /* a poster with pompos structure or "" */
} PILO_INIT_STR;


/* --
   -- Dynamics parameters
   --                                                                */
typedef struct PILO_DYNAMIC_PARAMS {
  int vMaxPercent;                   /* Vitesse lineaire max */
  int wMaxPercent;                   /* Vitesse angulaire max */
  int accelLinPercent;               /* Accel lin max */
  int accelAngPercent;               /* Accel ang max */
} PILO_DYNAMIC_PARAMS;


/* --
   -- Avoid or not 
   --                                                                */
typedef enum PILO_AVOID_FLAG {
  PILO_AVOID_OFF,
  PILO_AVOID_ON
} PILO_AVOID_FLAG;


/* --
   -- Translation 
   --                                                                */
typedef struct  PILO_MOVE_STR{
  double dL;                         /* Distance signee (m) */
  int vPercent;                      /* Vitesse max (m/s) */
  int accelPercent;                  /* Acceleration max (m/s2) */
} PILO_MOVE_STR;


/* --
   -- Arc of circle 
   --                                                                */
typedef struct PILO_TURN_STR {
  double dTheta;           /* Variation angulaire signee (>0:trigo) (rad) */
  double rayon;            /* Rayon signe (>0:centre giration a gauche) (m) */
  int wPercent;         /* Vitesse angulaire max (rad/s) */
  int accelPercent;        /* Accel angulaire max (rd/s2) */
} PILO_TURN_STR;


/* --
   -- Cartesian configution to reach
   --                                                                */

/* Local or global references */
typedef enum PILO_ABS_OR_REL {
  PILO_RELATIVE_TRAJ,        /* relative: init pos = current pos */
  PILO_ABSOLUTE_TRAJ         /* absolute: init pos = given pos
			   -> deviation smoothed if under the given margin */
} PILO_ABS_OR_REL; 

typedef enum PILO_GOTO_CONFIG_MODE {
  PILO_SPIN_TOWARD_POSITION,  /* Just spin the robot to the direction of the config */
  PILO_GOTO_POSITION,  /* Spin and translate the robot to the given pos */
  PILO_GOTO_CONFIG     /* Spin and translate the robot to the given pos, 
			  and turn it according to the final absolute given theta */
} PILO_GOTO_CONFIG_MODE;

typedef struct PILO_GOTO_CONFIG_STR {
  PILO_GOTO_CONFIG_MODE gotoConfigMode;  /* reach or not the final config */
  PILO_ABS_OR_REL absOrRelConfig;
  int speedPercent;    /* a value in [0..100] */
  int accelPercent;
  double x, y, theta;  /* the final configuration or position to reach */
} PILO_GOTO_CONFIG_STR;


/* --
   -- Poster name for an input trajectory
   --                                                                */
typedef struct PILO_TRAJ_POSTER {
  char posterName[64];
} PILO_TRAJ_POSTER;


/* --
   -- Trajectory 
   --                                                                */

/* Maximum number of reference points */
#define PILO_MAX_TRAJ_SEGMENTS     50
#define PILO_TRAJ_MAX_SEGMENTS PILO_MAX_TRAJ_SEGMENTS

/* Type of trajectory */
typedef enum PILO_SEG_TYPE {
  PILO_LINE_SEG,                    /* Ligne brisee (spin, trans, ...) */
  PILO_ARC_SEG                      /* Succession de droites et cercles */
} PILO_SEG_TYPE;

/* Header */
typedef struct PILO_TRAJ_HEADER {
  int numTraj;                     /* Trajectory identification */
  int nbSegs;                      /* Number of segments */
  PILO_SEG_TYPE segType;           /* broken line or arcs */
  PILO_ABS_OR_REL absOrRel;         /* relative or absolute */
  double x0, y0, theta0;           /* Initial position */
  double maxInitAngleDev;          /* Max deviation on the orientation */
  double maxInitLengthDev;         /* Max deviation on the position */
} PILO_TRAJ_HEADER;

/* One segment */
typedef struct PILO_SEGMENT {
  double dTheta;                /* Variation angulaire (rad) */
  double dL;                    /* puis variation lineaire (m) */
  double e;                     /* Tolerance d'ecart pour le lissage (m) */
} PILO_SEGMENT;

/* The trajectorory */
typedef struct PILO_TRAJECTORY {
  PILO_TRAJ_HEADER header;
  PILO_SEGMENT seg[PILO_MAX_TRAJ_SEGMENTS]; 
} PILO_TRAJECTORY;


/* --
   -- Absolute broken line (WILL DISAPPEARED)
   --                                                                */

/* Un point de la ligne brisee specifiee en absolue */
typedef struct PILO_ASOLUTE_REF {
  int forward;               /* 1: Marche avant, -1: amrche arriere */
  int trigo;                 /* 1: trigo, -1: inverse */
  double x;                  /* Abscisse du point (m) */
  double y;                  /* Ordonnee du point (m) */
  double e;                  /* Ecart tolere pour le lissage (m) */
} PILO_ABSOLUTE_REF;

/* Ligne brisee definie en global */
typedef struct PILO_ABSOLUTE_BL {
  PILO_TRAJ_HEADER header;
  double finalDTheta;        /* A last pure rotation */
  PILO_ABSOLUTE_REF point[PILO_MAX_TRAJ_SEGMENTS]; /* Tableau des points */
} PILO_ABSOLUTE_BL;


#endif
